// Planet system C version

// New version 2022, cleaned up.

#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils3.h"
#include "LittleOBJLoader.h"
#include <stdlib.h>
	//uses framework Cocoa
	//uses framework OpenGL

mat4 projectionMatrix;

Model *teddy, *sphere;

// Reference to shader programs
GLuint phongShader;

void init(void)
{
	// GL inits
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	printError("GL inits");

	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 300.0);

	// Load and compile shader
	phongShader = loadShaders("phong.vert", "phong.frag");
	printError("init shader");
	
	// Upload geometry to the GPU:
	teddy = LoadModelPlus("teddy.obj"); // Bunny with texture coords
	sphere = LoadModelPlus("sphere.obj"); // Bunny with texture coords

// Important! The shader we upload to must be active!
	glUseProgram(phongShader);
	glUniformMatrix4fv(glGetUniformLocation(phongShader, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	
	printError("init arrays");
}

GLfloat a = 0.0;

void display(void)
{
	printError("pre display");

	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	mat4 worldToView, m; // m1, m2, m, tr, scale;
	
	worldToView = lookAt(0,0,4, 0,0,0, 0,1,0);
	
	a += 0.03;

	mat4 A, B, C, D, E;
	float xs = 0.0;
	float ys = 0.0;
	float zs = 0.0;
	float b = -a*2;
	float c = a*5;
	float d = a*15;
	float rp = 2;
	float rm = 0.5;
	
	mat4 CAM = lookAt(0,4,4, 0,0,0, 0,1,0);
	A = Mult(CAM, T(xs, ys, zs));
	B = Mult(A, Mult(Ry(a), T(rp, 0, 0))); // Planet position
	C = Mult(B, Mult(Ry( c ), T(rm, 0, 0))); // Moon position
	D = Mult(B, Mult(Ry(b), S(0.3, 0.3, 0.3))); // Planet pos + rotation
	E = Mult(C, Mult(Ry(d), S(0.2, 0.2, 0.2))); // Moon pos + rotation

	glUseProgram(phongShader);
	glUniformMatrix4fv(glGetUniformLocation(phongShader, "modelviewMatrix"), 1, GL_TRUE, A.m);
	DrawModel(sphere, phongShader, "inPosition", "inNormal", NULL);
	
	glUseProgram(phongShader);
	glUniformMatrix4fv(glGetUniformLocation(phongShader, "modelviewMatrix"), 1, GL_TRUE, D.m);
	DrawModel(sphere, phongShader, "inPosition", "inNormal", NULL);
	
	glUseProgram(phongShader);
	glUniformMatrix4fv(glGetUniformLocation(phongShader, "modelviewMatrix"), 1, GL_TRUE, E.m);
	DrawModel(teddy, phongShader, "inPosition", "inNormal", NULL);

	
	printError("display");
	
	glutSwapBuffers();
}

void keys(unsigned char key, int x, int y)
{
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitContextVersion(3, 2);
	glutCreateWindow ("Planet system");
	glutRepeatingTimerFunc(20);
	glutDisplayFunc(display); 
	glutKeyboardFunc(keys);
	init ();
	glutMainLoop();
}
